##################################################################
##****************************************************************
##* Copyright (C) 1998 Sylvio Triebel
##*
##* $Id: editmodes.tcl,v 1.5 1998/01/28 16:33:56 silvio Exp $
##* Revision     : $Revision: 1.5 $
##* Datum        : 16.1.98
##* Systems      : Tcl/Tk 7.6 8.0, [incr tcl] 
##*
##*
##* Documentation: 
##*
##****************************************************************

#*********************** Interface *******************************

#Namen der Editmode-Prozeduren
set tkdraw_bind(rect)   tkdrawBindRect
set tkdraw_bind(oval)   tkdrawBindOval
set tkdraw_bind(region) tkdrawBindRegion
set tkdraw_bind(select) tkdrawBindSelect
set tkdraw_bind(move)   tkdrawBindMove
set tkdraw_bind(delete) tkdrawBindDelete
set tkdraw_bind(alter)  tkdrawBindAlter
set tkdraw_bind(line)   tkdrawBindLine
set tkdraw_bind(curve)  tkdrawBindCurve
set tkdraw_bind(poly)   tkdrawBindPolygon
set tkdraw_bind(spoly)  tkdrawBindSPoly
set tkdraw_bind(text)   tkdrawBindText

set tkdraw(oldx) -1
set tkdraw(oldy) -1

proc tkdrawBindEditmode { editmode canvas } {
    global tkdraw tkdraw_bind tkdraw_editOld

    $tkdraw_bind($editmode) $canvas

    if { $editmode != "text" } {
	tkdrawSetGlobalKeybindings
    }

    set tkdraw_editOld($canvas) $tkdraw(edit)
}

proc tkdrawLeaveEditmode { canvas } {
    global tkdraw tkdraw_bind tkdraw_editOld

    catch { set editmode $tkdraw_editOld($canvas) }
    catch { $tkdraw_bind($editmode)Destructor $canvas }
}

proc tkdrawRestoreEditmode { canvas } {
    global tkdraw tkdraw_bind tkdraw_editOld

    catch { 
	set editmode $tkdraw_editOld($canvas)
	$tkdraw_bind($editmode) $canvas 
	if { $editmode != "text" } {
# 	    puts "reset Keybindings"
	    tkdrawSetGlobalKeybindings
	}
    }
}

#*********************** Default ***************************************

proc tkdrawBindDefault { canvas } {
    tkdrawBindSelect $canvas
}

#*********************** Delete ****************************************

proc tkdrawBindDelete { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas

    #alle alten bindings loeschen
    tkdrawInitCanvasBindings

    bind $tkdraw(c) <ButtonPress-1> { 
	set tkdraw(oldx) [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdraw(oldy) [$tkdraw(c) canvasy %y $tkdraw(raster)]

	set id [$tkdraw(c) find closest $tkdraw(oldx) $tkdraw(oldy)]
	if { [tkdrawIsSelected $id] != 1 } {
	    tkdrawDeselectAll
	    tkdrawSelectItem $id
	}

	$tkdraw(c) delete selected
    }
}

#*********************** RECTANGLE ****************************************
proc tkdrawBindRect { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas
    set tkdraw(objectType) rect

    #alle alten bindings loeschen
    tkdrawInitCanvasBindings

    bind $tkdraw(c) <ButtonPress-1> { 
	tkdrawMarkRect $tkdrawX $tkdrawY $tkdraw(objectType)
    }
    bind $tkdraw(c) <ButtonRelease-1> {
	if { $tkdraw(regionact) } {
	    tkdrawMarkRect $tkdrawX $tkdrawY $tkdraw(objectType)
	}
    } 
    bind $tkdraw(c) <ButtonPress-3> { 
	if { $tkdraw(regionact) } {
	    tkdrawMarkRect $tkdrawX $tkdrawY $tkdraw(objectType)
	} else {
	    tkdrawBindMove $tkdrawX $tkdrawY
	}
    }
    bind $tkdraw(c) <Motion> { 
	set tkdrawX [eval $tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [eval $tkdraw(c) canvasy %y $tkdraw(raster)]
	tkdrawResizeRect $tkdrawX $tkdrawY
    }
}


proc tkdrawBindRectDestructor { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas

    #finish current object
    if { $tkdraw(regionact) } {
	tkdrawMarkRect $tkdraw(x) $tkdraw(y) $tkdraw(objectType)
    }
}

proc tkdrawMarkRect { x y what } {
    global tkdraw

    if { ($tkdraw(oldx) == $x) || ($tkdraw(oldy) == $y) } {
	return
    }

    if { $tkdraw(regionact) == 0 } {
	tkdrawDeselectAll
	set tkdraw(oldx) $x
	set tkdraw(oldy) $y
	set tkdraw(regionact) 1

	#Backgroundobject
	$tkdraw(c) create $what $x $y $x $y \
		-outline $tkdraw(color) \
		-fill $tkdraw(bgcolor) \
		-width $tkdraw(lwidth) \
		-tags "item bg grp endgrp resi"

	#Foregroundobject
	set tkdraw(current) [$tkdraw(c) create $what $x $y $x $y \
				 -outline $tkdraw(color) \
				 -fill $tkdraw(fill) \
				 -stipple $tkdraw(stipple) \
				 -width $tkdraw(lwidth) \
				 -tags "item grp endgrp resi" ]
    } else {
	set tkdraw(regionact) 0
	foreach id [$tkdraw(c) find withtag resi] {
	    $tkdraw(c) coords $id $tkdraw(oldx) $tkdraw(oldy) $x $y
	}
	$tkdraw(c) dtag resi resi
	tkdrawDeselectAll
	tkdrawSelectItem $tkdraw(current)
	set tkdraw(lastselected) $tkdraw(current)
	set tkdraw(oldx) -1
	set tkdraw(oldy) -1
	set tkdraw(undo) "tkdrawDeselectAll; tkdrawDeleteItem $tkdraw(current)"
    }
}

proc tkdrawResizeRect { x y } {
    global tkdraw
    
    if { $tkdraw(regionact) == 1 } {
	foreach id [$tkdraw(c) find withtag resi] {
	    $tkdraw(c) coords $id $tkdraw(oldx) $tkdraw(oldy) $x $y
	}
    }
}

#*********************** Oval ****************************************

proc tkdrawBindOval { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas

    tkdrawBindRect $canvas
    set tkdraw(objectType) oval
}

proc tkdrawBindOvalDestructor { canvas } {
    tkdrawBindRectDestructor $canvas
}

#*********************** Line ****************************************
proc tkdrawBindLine { canvas } {
    global tkdraw

    set tkdraw(canvas) $tkdraw(c)

    #alle alten bindings loeschen
    tkdrawInitCanvasBindings

    set tkdraw(smooth) false
    set tkdraw(coords) ""
    set tkdraw(polygon) 0
    set tkdraw(moveactiv) 0

    bind $tkdraw(c) <ButtonPress-1> { 
	set tkdraw(moveactiv) 1
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	set tkdraw(oldx) $tkdrawX
	set tkdraw(oldy) $tkdrawY

	set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-3]] $tkdrawX $tkdrawY $tkdrawX $tkdrawY "	

	tkdrawUpdateLine
    }
    bind $tkdraw(c) <Shift-ButtonPress-1> { 
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	if { $tkdraw(moveactiv) } {
	    if { [llength $tkdraw(coords)] > 4 } {
		set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-5]] $tkdrawX $tkdrawY "	
		tkdrawUpdateLine
	    }
	}
    }
    bind $tkdraw(c) <Motion> { 
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	if { $tkdraw(moveactiv) == 1 } {
	    set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-3]] $tkdrawX $tkdrawY "
	    tkdrawUpdateLine
	}
    }
    bind $tkdraw(c) <ButtonPress-3> {
	if { $tkdraw(moveactiv) } {
	    $tkdraw(c) delete withtag temp
	    tkdrawUpdateLine
	    set tkdraw(moveactiv) 0
	    set tkdraw(coords) ""
	    tkdrawDeselectAll
	    tkdrawSelectItem $tkdraw(polygon)   
	    set tkdraw(lastselected) $tkdraw(polygon)
	    $tkdraw(c) dtag resi resi
	    set tkdraw(undo) "tkdrawDeselectAll; tkdrawDeleteItem $tkdraw(polygon)"
	    set tkdraw(polygon) 0
	} else {
	    set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	    set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	    tkdrawBindMove $tkdrawX $tkdrawY
	}
    } 
}

proc tkdrawUpdateLine { } {
    global tkdraw

    if { $tkdraw(polygon) == 0 } {
	tkdrawDeselectAll
	#Backgroundobject
	eval $tkdraw(c) create line \
		$tkdraw(coords) -fill \"$tkdraw(bgcolor)\" \
		-width $tkdraw(lwidth) \
		-arrow $tkdraw(arrow) \
		-tags \"item bg grp endgrp resi\" -smooth $tkdraw(smooth)

	#Foregroundobject
	set tkdraw(polygon) [ eval $tkdraw(c) create line \
		$tkdraw(coords) -fill \"$tkdraw(color)\" \
		-width $tkdraw(lwidth) \
		-arrow $tkdraw(arrow) \
		-stipple \"$tkdraw(stipple)\" \
		-tags \"item grp endgrp resi\" \
		-smooth $tkdraw(smooth)]
    }
    foreach id [$tkdraw(c) find withtag resi] {
	eval $tkdraw(c) coords $id $tkdraw(coords)
    }
}

#*********************** Curve ****************************************
proc tkdrawBindCurve { canvas } {
    global tkdraw

    tkdrawBindLine $canvas
    set tkdraw(smooth) 1
}

proc tkdrawBindCurveDestructor { canvas } {
    tkdrawBindLineDestructor $canvas
}

#*********************** Polygon ****************************************
proc tkdrawBindPolygon { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas

    #alle alten bindings loeschen
    tkdrawInitCanvasBindings

    set tkdraw(smooth) false
    set tkdraw(coords) ""
    set tkdraw(polygon) 0
    set tkdraw(moveactiv) 0

    bind $tkdraw(c) <ButtonPress-1> { 
	set tkdraw(moveactiv) 1
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	set tkdraw(oldx) $tkdrawX
	set tkdraw(oldy) $tkdrawY

	set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-3]] $tkdrawX $tkdrawY $tkdrawX $tkdrawY "	

	tkdrawUpdatePolygon
    }
    bind $tkdraw(c) <Shift-ButtonPress-1> { 
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	if { $tkdraw(moveactiv) } {
	    if { [llength $tkdraw(coords)] > 4 } {
		set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-5]] $tkdrawX $tkdrawY "	
		tkdrawUpdatePolygon
	    }
	}
    }
    bind $tkdraw(c) <Motion> { 
	set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	if { $tkdraw(moveactiv) == 1 } {
	    set tkdraw(coords) "[lrange $tkdraw(coords) 0 [expr [llength $tkdraw(coords)]-3]] $tkdrawX $tkdrawY "
	    tkdrawUpdatePolygon
	}
    }
    bind $tkdraw(c) <ButtonPress-3> {
	if { $tkdraw(moveactiv) == 1 } {
	    tkdrawUpdatePolygon
	    set tkdraw(moveactiv) 0
	    set tkdraw(coords) ""
	    tkdrawDeselectAll
	    catch { tkdrawSelectItem $tkdraw(polygon) }   
	    set tkdraw(lastselected) $tkdraw(polygon)
	    $tkdraw(c) dtag resi resi
	    $tkdraw(c) dtag selbox temp
	    $tkdraw(c) delete withtag temp
	    $tkdraw(c) addtag temp withtag selbox
	    set tkdraw(undo) "tkdrawDeselectAll; catch { tkdrawDeleteItem $tkdraw(polygon) }"
	    set tkdraw(polygon) 0
	} else {
	    set tkdrawX [$tkdraw(c) canvasx %x $tkdraw(raster)]
	    set tkdrawY [$tkdraw(c) canvasy %y $tkdraw(raster)]
	    tkdrawBindMove $tkdrawX $tkdrawY
	}
    } 
}

proc tkdrawBindPolygonDestructor { canvas } {
    global tkdraw

    $canvas dtag selbox temp
    $canvas delete withtag temp
    $canvas addtag temp withtag selbox
}

proc tkdrawUpdatePolygon { } {
    global tkdraw

    if { [llength $tkdraw(coords)] >= 6 } {
	$tkdraw(c) delete withtag temp
	if { $tkdraw(polygon) == 0 } {
	    #Backgroundobject
	    eval $tkdraw(c) create polygon \
		    $tkdraw(coords) -fill \"$tkdraw(bgcolor)\" \
		    -outline \"$tkdraw(bgcolor)\" \
		    -width $tkdraw(lwidth) \
		    -tags \"item bg grp endgrp resi\" -smooth $tkdraw(smooth)
	    #Foregroundobject	
	    set tkdraw(polygon) [eval $tkdraw(c) create polygon \
		    $tkdraw(coords) -fill \"$tkdraw(fill)\" \
		    -outline \"$tkdraw(color)\" \
		    -width $tkdraw(lwidth) \
		    -stipple \"$tkdraw(stipple)\" \
		    -tags \"item grp endgrp resi\" -smooth $tkdraw(smooth) ]
	}
	foreach id [$tkdraw(c) find withtag resi] {
	    eval $tkdraw(c) coords $id $tkdraw(coords)
	}
    } else {
	tkdrawDeselectAll
	catch { $tkdraw(c) delete $tkdraw(polygon) }
	catch { $tkdraw(c) delete withtag temp }
	set tkdraw(polygon) 0
	set tkdraw(tmpline) [$tkdraw(c) create line 0 0 0 0 \
		-width $tkdraw(lwidth) \
		-tags temp -fill black]
	eval $tkdraw(c) coords $tkdraw(tmpline) $tkdraw(coords)
    }
}

#*********************** Curve ****************************************
proc tkdrawBindSPoly { canvas } {
    global tkdraw

    tkdrawBindPolygon $canvas
    set tkdraw(smooth) true
}

proc tkdrawBindSPolyDestructor { canvas } {
    tkdrawBindPolygonDestructor $canvas
}




