##################################################################
##*****************************************************************
##* Copyright (C) 1998 Sylvio Triebel
##*
##* Id           : $Id: editprocs.tcl,v 1.5 1998/01/28 16:33:56 silvio Exp $
##* Revision     : $Revision: 1.5 $
##* Datum        : 16.1.98
##* Systems      : Tcl/Tk 7.6 8.0, [incr tcl] 
##*
##*
##* Documentation: 
##*
##*****************************************************************


proc tkdrawCopySelected {} {
    global tkdraw
   
    clipboard clear -displayof . 
    foreach id [$tkdraw(canvas) find withtag selected] {
	if { [tkdrawIsItem $id] == 1 } {
	    set obj_str [tkdrawItemToString $id]
	    clipboard append -displayof . "\{ $obj_str \} "
	}
    }
}

proc tkdrawCutSelected {} {
    global tkdraw

    tkdrawCopySelected
    tkdrawDeleteSelected
}

proc tkdrawPaste {} {
    global tkdraw

    #zur sicherheit
    tkdrawResetGroupMap

    tkdrawDeselectAll
    foreach str [selection get  -displayof . -selection CLIPBOARD] {
	tkdrawStringToItem $str $tkdraw(canvas)
    }

    $tkdraw(canvas) move selected \
	[expr 2*$tkdraw(raster)] [expr 2*$tkdraw(raster)]

    #reset groupmap after each import procedure!
    tkdrawResetGroupMap

    set id [lindex [$tkdraw(canvas) find withtag selected] 1]
    tkdrawSelectItem $id

    set tkdraw(undo) ""
}

proc tkdrawDeleteSelected {} {
    global tkdraw
    $tkdraw(canvas) delete withtag selected

    set tkdraw(undo) ""
}

proc tkdrawDeleteItem { id } {
    global tkdraw
    $tkdraw(canvas) delete withtag [expr $id -1]
    $tkdraw(canvas) delete withtag $id

    set tkdraw(undo) ""
}

proc tkdrawDuplicateSelected {} {
    tkdrawCopySelected
    tkdrawPaste
}

proc tkdrawRotateSelected { angle } {
    global tkdraw

    set tkdraw(undo) ""
}

proc tkdrawRiseSelected {} {
    global tkdraw

    foreach id [$tkdraw(canvas) find withtag selected] {
	    $tkdraw(canvas) raise $id
    }

    set tkdraw(undo) ""
}

proc tkdrawLowerSelected {} {
    global tkdraw

    set l  [$tkdraw(canvas) find withtag selected]

    #reverse list
    for { set idx [expr [llength $l]-1] } { $idx >= 0 } { incr idx -1 } {
	set id [lindex $l $idx]
	$tkdraw(canvas) lower $id
    }

    set tkdraw(undo) ""
}


proc tkdrawIsItem { id } {
    global tkdraw

    if { [lsearch [$tkdraw(canvas) gettags $id] item] == -1 } {
	return 0
    }

    return 1
}

proc tkdrawIsBgItem { id } {
    global tkdraw

    if { [lsearch [$tkdraw(canvas) gettags $id] bg] == -1 } {
	return 0
    }
    return 1
}

proc tkdrawInitCanvasBindings { } {
    global tkdraw

    tkdrawClearGlobalKeybindings

    #alle alten bindings loeschen
    foreach b [bind $tkdraw(canvas)] {
	if { $b != "<Enter>" } {
	    bind $tkdraw(canvas) $b "" 
	}
    }
}


set tkdraw(currentcmd) ""
proc tkdrawAddCommandSelected { } {
    global tkdraw

    catch { destroy .cmded  }

    entry .cmded -textvariable tkdraw(currentcmd) \
	-relief sunken -borderwidth 1 -justify center -width 20 \
	-background gray

    bind .cmded <Return> {
	$tkdraw(canvas) delete $tkdraw(cureditor)
	tkdrawUpdateCmdSelected
	tkdrawSetGlobalKeybindings
    }

    set bbox [$tkdraw(canvas) bbox selected]
    set xm [expr ([lindex $bbox 0]+[lindex $bbox 2])/2]
    set ym [expr ([lindex $bbox 1]+[lindex $bbox 3])/2]

    set tkdraw(cureditor) [$tkdraw(canvas) create window $xm $ym \
	    -window .cmded ]

    tkdrawClearGlobalKeybindings
}

proc tkdrawUpdateCmdSelected { } {
    global tkdraw

    foreach id [$tkdraw(canvas) find withtag selected] {
	set tags [$tkdraw(canvas) gettags $id]
	if { [lsearch $tags bg] != -1 } continue

	if { [lsearch $tags cmd] == -1 } {
	    set tags "cmd {} $tags"
	}
	set idx [expr [lsearch $tags cmd]+1]
	set cmd $tkdraw(currentcmd)
	set tags [lreplace $tags $idx $idx $cmd ]
	
	$tkdraw(canvas) itemconfigure $id -tags $tags
    }    

    set tkdraw(undo) ""
}

proc tkdrawFlipxSelected { } {
    global tkdraw

    set bbox [$tkdraw(canvas) bbox selected]
    set xm [expr ([lindex $bbox 0]+[lindex $bbox 2])/2]

    if { $tkdraw(gravity) } {
	#snap xm, ym to grid!
	set xm [expr round($xm/$tkdraw(raster))*$tkdraw(raster)]
    }

    foreach id [$tkdraw(canvas) find withtag selected] {
	set nco [tkdrawTransformMY $xm [$tkdraw(canvas) coords $id]]
	eval $tkdraw(canvas) coords $id $nco
    }    

    set tkdraw(undo) "tkdrawFlipxSelected"
}

proc tkdrawFlipySelected { } {
    global tkdraw

    set bbox [$tkdraw(canvas) bbox selected]
    set ym [expr ([lindex $bbox 1]+[lindex $bbox 3])/2]

    if { $tkdraw(gravity) } {
	#snap xm, ym to grid!
	set ym [expr round($ym/$tkdraw(raster))*$tkdraw(raster)]
    }

    foreach id [$tkdraw(canvas) find withtag selected] {
	set nco [tkdrawTransformMX $ym [$tkdraw(canvas) coords $id]]
	eval $tkdraw(canvas) coords $id $nco
    }    

    set tkdraw(undo) "tkdrawFlipySelected"
}

proc tkdrawRotateSelected { alpha } {
    global tkdraw

    set bbox [$tkdraw(canvas) bbox selected]
    set xm [expr ([lindex $bbox 0]+[lindex $bbox 2])/2]
    set ym [expr ([lindex $bbox 1]+[lindex $bbox 3])/2]

    if { $tkdraw(gravity) } {
	#snap xm, ym to grid!
	set xm [expr round($xm/$tkdraw(raster))*$tkdraw(raster)]
	set ym [expr round($ym/$tkdraw(raster))*$tkdraw(raster)]
    }

    foreach id [$tkdraw(canvas) find withtag selected] {
	set nco [tkdrawTransformRotate $xm $ym $alpha [$tkdraw(canvas) coords $id]]
	eval $tkdraw(canvas) coords $id $nco
    }    

    set tkdraw(undo) "tkdrawRotateSelected -$alpha"
}

set tkdraw(sfak) "0.5 0.5"
proc tkdrawScaleSelectedDlg { } {
    global tkdraw

    set f .scale

    catch { destroy $f }

    toplevel $f
    wm title $f "Scale"
    wm iconname $f "Scale"

    label $f.label -text "ScaleX ScaleY" -width 12
    entry $f.entry -textvariable tkdraw(sfak)

    frame $f.b
    set tkdraw(bres) 0
    button $f.b.ok  -text Apply  -command "set tkdraw(bres) ok; destroy $f"
    button $f.b.can -text Cancel -command "set tkdraw(bres) esc; destroy $f"
    pack $f.b.ok $f.b.can -side left -padx 2m -pady 1m

    pack $f.label $f.entry -side top
    pack $f.b

    bind $f <Return> "set tkdraw(bres) ok; destroy $f"
    bind $f <Escape> "set tkdraw(bres) esc; destroy $f"

    tkwait visibility $f
    grab $f
    tkwait window $f

    if { $tkdraw(bres) == "ok" } {
	set bbox [$tkdraw(canvas) bbox selected]
	set xm [expr ([lindex $bbox 0]+[lindex $bbox 2])/2]
	set ym [expr ([lindex $bbox 1]+[lindex $bbox 3])/2]

	set scalex [lindex $tkdraw(sfak) 0]
	set scaley [lindex $tkdraw(sfak) 1]

	tkdrawScaleSelected $xm $ym $scalex $scaley
 
	set tkdraw(undo) "tkdrawScaleSelected $xm $ym [expr 1/$scalex] [expr 1/$scaley]"
    }   
}

proc tkdrawScaleSelected { xm ym scalex scaley } {
    global tkdraw

    foreach id [$tkdraw(canvas) find withtag selected] {
	if { [$tkdraw(canvas) type $id] == "text" } {
	    #scale font
	    tkdrawRescaleFont $id $scalex
	}
	set nco [tkdrawTransformScale $xm $ym $scalex $scaley \
		     [$tkdraw(canvas) coords $id]]
	eval $tkdraw(canvas) coords $id $nco
    } 
}


proc tkdrawTransformMY { x0 coords } {
    global tkdraw

    set ncoord [list]
    set isx 1
    foreach i $coords {
	if { $isx } {
	    lappend ncoord [expr 2*$x0-$i]
	    set isx 0
	} else { 
	    lappend ncoord $i
	    set isx 1
	}
    }
    return $ncoord
}

proc tkdrawTransformMX { y0 coords } {
    global tkdraw

    set ncoord [list]
    set isx 1
    foreach i $coords {
	if { $isx } {
	    lappend ncoord $i
	    set isx 0
	} else { 
	    lappend ncoord [expr 2*$y0-$i]
	    set isx 1
	}
    }
    return $ncoord
}


proc tkdrawTransformRotate { x0 y0 alpha coords } {
    global tkdraw

    set alpha [expr $alpha * -3.141592653589793/180]

    set ncoord [list]
    set isx 1
    
    foreach y $coords {
	if { $isx } {
	    set x $y
	    set isx 0
	} else { 
	    set xneu [expr $x0+($x-$x0)*cos($alpha)-($y-$y0)*sin($alpha)]
	    set yneu [expr $y0+($x-$x0)*sin($alpha)+($y-$y0)*cos($alpha)]
	    lappend ncoord $xneu $yneu
	    set isx 1
	}
    }
    return $ncoord
}


proc tkdrawTransformScale { x0 y0 scalex scaley coords } {
    global tkdraw

    set ncoord [list]
    set isx 1
    
    foreach y $coords {
	if { $isx } {
	    set x $y
	    set isx 0
	} else { 
	    set xneu [expr $x0+($x-$x0)*$scalex]
	    set yneu [expr $y0+($y-$y0)*$scaley]
	    lappend ncoord $xneu $yneu
	    set isx 1
	}
    }
    return $ncoord
}


proc tkdrawAlignSelected { {alignx 1} {aligny 1} } {
    global tkdraw

    #bbox des Referenzobjektes
    set bbox1 [$tkdraw(canvas) bbox withtag firstsel]

    set xa1  [expr [lindex $bbox1 0]+1]
    set ya1  [expr [lindex $bbox1 1]+1]
    set xe1  [expr [lindex $bbox1 2]-1]
    set ye1  [expr [lindex $bbox1 3]-1]
    set xm1 [expr ($xa1+$xe1)/2]
    set ym1 [expr ($ya1+$ye1)/2]

    set bbox2 [$tkdraw(canvas) bbox withtag secsel]

    set xa2  [expr [lindex $bbox2 0]+1-$tkdraw(objmargin)*$tkdraw(scaling)]
    set ya2  [expr [lindex $bbox2 1]+1-$tkdraw(objmargin)*$tkdraw(scaling)]
    set xe2  [expr [lindex $bbox2 2]-1+$tkdraw(objmargin)*$tkdraw(scaling)]
    set ye2  [expr [lindex $bbox2 3]-1+$tkdraw(objmargin)*$tkdraw(scaling)]
    set xm2  [expr ($xa2+$xe2)/2]
    set ym2  [expr ($ya2+$ye2)/2]

    switch -exact $tkdraw(refx) {
	center  { set xref [expr ($xa1+$xe1)/2]	}
	left    { set xref $xa1 }
	right   { set xref $xe1 }
	default { set xref [expr ($xa1+$xe1)/2] }
    }

    switch -exact $tkdraw(refy) {
	center  { set yref [expr ($ya1+$ye1)/2]	}
	top     { set yref $ya1 }
	bottom  { set yref $ye1 }
	default { set yref [expr ($ya1+$ye1)/2]	}
    }

    switch -exact $tkdraw(objx) {
	center  { set xobj [expr ($xa2+$xe2)/2]	}
	left    { set xobj $xa2 }
	right   { set xobj $xe2	}
	default { set xobj $xref }
    }

    switch -exact $tkdraw(objy) {
	center  { set yobj [expr ($ya2+$ye2)/2]	}
	top     { set yobj $ya2 }
	bottom  { set yobj $ye2	}
	default { set yobj $yref }
    }

    if { $alignx } {
	set dx [expr $xref-$xobj]
    } else {
	set dx 0
    }

    if { $aligny } {
	set dy [expr $yref-$yobj] 
    } else {
	set dy 0
    }

    
    $tkdraw(canvas) move secsel $dx $dy
    $tkdraw(canvas) move secselbox $dx $dy

    set tkdraw(undo) "$tkdraw(canvas) move secsel -$dx -$dy;$tkdraw(canvas) move secselbox -$dx -$dy"
}

proc tkdrawSetObjMargin { } {
    global tkdraw

    set f .objmargin

    catch { destroy $f }

    toplevel $f
    wm title $f "Margin"
    wm iconname $f "Margin"

    label $f.label -text "Object Margin"
    set tkdraw(tmpobjmargin) $tkdraw(objmargin)
    entry $f.entry -textvariable tkdraw(tmpobjmargin)

    frame $f.b
    set tkdraw(bres) 0
    button $f.b.ok  -text Apply  -command "set tkdraw(bres) ok; destroy $f"
    button $f.b.can -text Cancel -command "set tkdraw(bres) esc; destroy $f"
    pack $f.b.ok $f.b.can -side left

    pack $f.label $f.entry -side top -fill x
    pack $f.b

    tkwait visibility $f
    grab $f
    tkwait window $f

    if { $tkdraw(bres) == "ok" } {
	set tkdraw(objmargin) $tkdraw(tmpobjmargin)
    }   
}
