##################################################################
##*****************************************************************
##* Copyright (C) 1998 Sylvio Triebel
##*
##* Id           : $Id: io.tcl,v 1.5 1998/01/28 16:33:56 silvio Exp $
##* Revision     : $Revision: 1.5 $
##* Datum        : 16.1.98
##* Systems      : Tcl/Tk 7.6 8.0, [incr tcl] 
##*
##*
##* Documentation: 
##*
##*****************************************************************


set tkdraw(filename)    noname.tdr

proc tkdrawNewFile { {filename noname.tdr} } {
    global tkdraw

    set tkdraw(filename)    $filename

    $tkdraw(c) delete all
    tkdrawZoom 1.0
    $tkdraw(c) xview moveto 0
    $tkdraw(c) yview moveto 0
    tkdrawInitGroupManager

    if { $tkdraw(gravity) } {
	set tkdraw(raster) $tkdraw(rastersize)
    }
    set tkdraw(edit) select
    set tkdraw(modified) 0

}

proc tkdrawOpenFile { {filename 0} {par .}  } {
    global tkdraw

    if { $filename == 0 } { 
	set filename [tk_getOpenFile -parent $par \
		-filetypes { {{TKDraw Datei} {.tdr}} {{TKDraw Datei} {.eps}} }] 
    }

    if { $filename == 0 || $filename == "" } {
	tkdrawStatusMessage "No file selected"
	return
    }

    set tkdraw(filename) $filename

    tkdrawNewFile $tkdraw(filename)
 
    tkdrawParseFile $filename

    wm title . $filename
    wm iconname . $filename
}

proc tkdrawSaveFile { {keepfilename 0} {par .}  } {
    global tkdraw

    if { $keepfilename == 0 } { 
	set filename [tk_getSaveFile -parent $par \
		-filetypes { {{TKDraw Datei} {.tdr}} {{TKDraw Datei} {.eps}} } \
		-defaultextension .tdr ] 
	if { $filename != "" } {
	    set tkdraw(filename) $filename
	} else {
	    tkdrawStatusMessage "No file selected"
	    return
	}
    } else {
	set filename $tkdraw(filename)
    }

    tkdrawWriteFile $filename
    set tkdraw(modified) 0
}

proc tkdrawParseFile { filename } {
    global tkdraw

    set file [open $filename r]
    set parseflag "bad"

    while { ![eof $file] } {
	gets $file line

	catch {
	    set first [lindex $line 0]
	    switch -exact $first {
		"%item" {
		    #substitude short pathnames in -stipple
		    set idx [expr [lsearch $line "-stipple"]+1]
		    if { $idx > 0 } {
			set fname [tkdrawBmFullname [lindex $line $idx]]
			set line [lreplace $line $idx $idx $fname]
		    }

		    tkdrawStringToItem $line
		}
		"%set" {
		    set varname [lindex $line 1]
		    set value [lindex $line 2]
		    set tkdraw($varname) $value
		}
		"%Creator" {
		    if { [lindex $line 1] == "TKDraw" } {
			set tkdraw(documentversion) [lindex $line 2]
			
			set parseflag "ok"
		    }
		}
	    }
	}
    }
    close $file

    if { $parseflag != "ok" } {
	set tmp [tk_dialog .quitdlg "Error" \
		     "File $filename isn't a proper TKDraw file\nIf you continue data may be destroyed" \
		     error 0 Quit Continue]


	switch $tmp {
	    0 { 
		destroy .
	    }
	}
    }

    #reset groupmap after each import procedure!
    tkdrawResetGroupMap
}

proc tkdrawWriteFile { filename  {postscript 0} } {
    global tkdraw

    #erstmal aufraumen
    tkdrawDeselectAll
    $tkdraw(c) delete withtag temp
    $tkdraw(c) dtag current current

    set file [open $filename w]

    if { $tkdraw(postscript) } {
	#Postscript erzeugen
	tkdrawZoom 1.0
	set b [$tkdraw(c) bbox item]
	set w [expr [lindex $b 2] - [lindex $b 0]]
	set h [expr [lindex $b 3] - [lindex $b 1]]

	puts $file [$tkdraw(c) postscript \
		-colormode color \
		-width $w -height $h \
		-x [lindex $b 0] -y [lindex $b 1] \
		]
    }

    # Creator line
    set version [lindex [split $tkdraw(revision) " "] 1]
    puts $file "%Creator TKDraw $version"

    #Variablen speichern
    set varlist {
	gravity showgrid postscript
    }
    foreach var $varlist {
	puts $file "%set $var $tkdraw($var)" 
    }

    #Alle canvas-items rausschreiben
    foreach obj [$tkdraw(c) find withtag item] {
	set line [tkdrawItemToString $obj]
	
	#substitude full pathnames in -stipple
	set idx [expr [lsearch $line "-stipple" ] + 1 ]
	if { $idx > 0 } {
	    set fname [ tkdrawBmBasename [lindex $line $idx] ]
	    set line [lreplace $line $idx $idx $fname]
	}
	#substituded

	puts $file $line
    }
    close $file
}

set tkdraw(optionlist) {
    -tags
    -outline -fill -stipple 
    -width -arrow -smooth
    -text -font -justify -anchor 
}

proc tkdrawItemToString { obj } {
    global tkdraw

    set line "%item [$tkdraw(canvas) type $obj] "

    #quick hack
    if { "$tkdraw(canvas)" == "$tkdraw(c)" } {
	set scaling $tkdraw(scaling)
    } else {
	set scaling 1.0
    }

    foreach val [$tkdraw(canvas) coords $obj] {
	append line "[expr $val/$scaling] "
    }

    foreach conf [$tkdraw(canvas) itemconfigure $obj] {
	set option [lindex $conf 0]
	set value  [lindex $conf 4]
	set defval [lindex $conf 3]

	if { [lsearch $tkdraw(optionlist) $option] == -1 } {
	    continue
	}

# 	if { $value != "" } {
	#problem z. B. mit polygon :
	#-fill "" wird nicht herausgeschrieben
	if { $value != $defval } {
	    if { [llength $value] == 1 } {
		#because tk bug
		if { $value == "bezier" } {
		    set value 1
		}
		append line " $option $value"
	    } else {
		append line " $option \{$value\}"
	    }
	}
# 	}
    }

    return $line
}

proc tkdrawStringToItem { str {offset 0} {canvas ""} } {
    global tkdraw
	
    if { $canvas == "" } {
	set canvas $tkdraw(c)
    }

    set id [eval $canvas create [lrange $str 1 end]]
    tkdrawRemapGroups $id

    #quick hack 
    if { "$canvas" == "$tkdraw(c)" } {
	$tkdraw(c) scale $id 0 0 $tkdraw(scaling) $tkdraw(scaling)
    }

    return $id
}

proc tkdrawBmBasename { file } {
    global tkdraw

    return [lindex [file split $file] end]
}

proc tkdrawBmFullname { file } {
    global tkdraw

    return @$tkdraw(BITMAP)/$file
}
