##################################################################
##****************************************************************
##* Copyright (C) 1998 Sylvio Triebel
##*
##* $Id: select.tcl,v 1.6 1998/02/02 13:33:24 silvio Exp $
##* Revision     : $Revision: 1.6 $
##* Datum        : 16.1.98
##* Systems      : Tcl/Tk 7.6 8.0, [incr tcl] 
##*
##*
##* Documentation: 
##*
##****************************************************************


#*********************** Select/Move *****************************
set tkdraw(lastselected) ""

proc tkdrawBindSelect { canvas } {
    global tkdraw

    set tkdraw(canvas) $canvas

    set tkdraw(moveactiv) 0
    set tkdraw(firstselected) 1

    tkdrawInitCanvasBindings

    bind $tkdraw(canvas) <ButtonPress-1> {
	set id [$tkdraw(canvas) find withtag current]
	if { $id != "" } {
	    tkdrawDeselectAll
	    tkdrawSelectItem $id
	} else {
	    # daneben!
	    if { [$tkdraw(canvas) find withtag selected] == "" } {
		tkdrawDeselectAll
		#Bereich selektieren
		lappend tkdraw(regionhook) "tkdrawSelectRegion \$tkdraw(region)"

		tkdrawBindRegion 1 $tkdrawX $tkdrawY
	    } else {
		#first klick to deselect all !
		tkdrawDeselectAll
	    }
	}
    }

    bind $tkdraw(canvas) <Double-Button-1> {
	set id [$tkdraw(canvas) find withtag current]
	if { $id != "" } {
	    set tags [$tkdraw(canvas) gettags $id]
	    set idx [expr [lsearch $tags cmd]+1]
	    set cmd [lindex $tags $idx]
	    if { $idx && $cmd != "" } {
		eval $cmd
	    }
	}
    }
    bind $tkdraw(canvas) <Shift-ButtonPress-1> {
	set id [$tkdraw(canvas) find withtag current]

	if { $id != "" } {
	    if { [tkdrawIsSelected $id] == 1 } {
		tkdrawDeselectItem $id
	    } else {
		tkdrawSelectItem $id
	    }
	} else {
		#Bereich selektieren
		lappend tkdraw(regionhook) "tkdrawSelectRegion \$tkdraw(region)"

		tkdrawBindRegion 1 $tkdrawX $tkdrawY
	}
    }

    bind $tkdraw(canvas) <ButtonPress-3> { 
	set tkdraw(oldx) $tkdrawX
	set tkdraw(oldy) $tkdrawY

	set id [$tkdraw(canvas) find closest $tkdraw(oldx) $tkdraw(oldy)]
	if { [tkdrawIsSelected $id] != 1 } {
	    tkdrawDeselectAll
	    tkdrawSelectItem $id
	}
	set tkdraw(moveactiv) 1
    }

    bind $tkdraw(canvas) <ButtonRelease-3> { 
	set tkdraw(moveactiv) 0
    }

    bind $tkdraw(canvas) <Motion> {
	set tkdrawX [$tkdraw(canvas) canvasx %x $tkdraw(raster)] 
	set tkdrawY [$tkdraw(canvas) canvasy %y $tkdraw(raster)]
	if { $tkdraw(moveactiv) } {
	    set dx [expr $tkdrawX - $tkdraw(oldx)]
	    set dy [expr $tkdrawY - $tkdraw(oldy)]
	    set tkdraw(oldx) $tkdrawX
	    set tkdraw(oldy) $tkdrawY
	    $tkdraw(canvas) move selected $dx $dy
	}
    }
}

proc tkdrawBindSelectDestructor { } {
    global tkdraw

    set tkdraw(lastselected) [$tkdraw(canvas) find withtag selected]
    tkdrawDeselectAll
}

proc tkdrawSelectItem { id } {
    global tkdraw

    if { $id != "" && ([tkdrawIsItem $id] == 1) } {
	if { [lsearch [$tkdraw(canvas) gettags $id] selected] != -1 } {
	    return ""
	    #wurde schon selektiert!
	}

	if { [lsearch [$tkdraw(canvas) gettags $id] bg] != -1 } {
	    set top [expr $id+1]
	    return [tkdrawSelectItem $top]
	}
	
	if { ! $tkdraw(modified) } { set tkdraw(modified) 1 }

	set xtags ""
	set groups [tkdrawGetGroups $id]

	#fuer align wichtig!
	if { $tkdraw(firstselected) } {
	    set tkdraw(firstselected) 0
	    set selmod firstsel
	} else {
	    set selmod secsel
	}

	if { $groups == "" } {
	    #Einzelobjekt
	    $tkdraw(canvas) addtag selected withtag $id
# 	    $tkdraw(canvas) addtag selected below $id
	    #this is done because an error of dash patch
	    $tkdraw(canvas) addtag selected withtag [expr $id-1]

	    $tkdraw(canvas) addtag $selmod withtag $id
	    $tkdraw(canvas) addtag $selmod withtag [expr $id-1]

	    set bbox [$tkdraw(canvas) bbox $id]
	    set xtags id$id
	} else {
	    #Gruppenobjekt
	    foreach group $groups {
		$tkdraw(canvas) addtag selected withtag $group
		$tkdraw(canvas) addtag lastselected withtag $group
		$tkdraw(canvas) addtag $selmod withtag $group
		append xtags "gr$group "
	    }
	    set bbox [$tkdraw(canvas) bbox lastselected]
	    $tkdraw(canvas) dtag lastselected lastselected
	}

	eval $tkdraw(canvas) create rectangle $bbox -outline red -width 3 \
	    -tags \"selbox temp selected ${selmod}box $xtags\"

	return $bbox
    }

    return ""
}

proc tkdrawDeselectItem { id } {
    global tkdraw

    if { $id == "" } return

    if { [tkdrawIsItem $id] } {
	if { [lsearch [$tkdraw(canvas) gettags $id] bg] != -1 } {
	    set top [expr $id+1]
	    return [tkdrawDeselectItem $top]
	}

	$tkdraw(canvas) dtag $id selected
	$tkdraw(canvas) dtag [expr $id-1] selected

	set groups [tkdrawGetGroups $id]
	
	if { $groups == "" } {
	    #Einzelobjekt
	    $tkdraw(canvas) delete withtag id$id
	} else {
	    #Gruppenobjekt
	    foreach group $groups {
		$tkdraw(canvas) dtag $group selected
		$tkdraw(canvas) dtag $group lastselected
		catch { $tkdraw(canvas) delete withtag gr$group }
	    }
	    $tkdraw(canvas) dtag lastselected lastselected
	}
    } elseif { [lsearch [$tkdraw(canvas) gettags $id] selbox] != -1 } {
	set tags [$tkdraw(canvas) gettags $id]

	foreach tag $tags {
	    switch -glob $tag {
		id* {
		    set o [string range $tag 2 end]
		    $tkdraw(canvas) dtag $o selected
		    $tkdraw(canvas) dtag [expr $o-1] selected
		}
		gr* {
		    set group [string range $tag 2 end]
		    $tkdraw(canvas) dtag $group selected
		    $tkdraw(canvas) dtag $group lastselected	
		}
	    }
	}
	#selectbox loeschen
	$tkdraw(canvas) delete withtag $id
    }
}

proc tkdrawSelectRegion { region } {
    global tkdraw

    eval $tkdraw(canvas) dtag regsel regsel
    eval $tkdraw(canvas) addtag regsel enclosed $region

    set x0 [lindex $region 0]
    set y0 [lindex $region 1]
    set x1 [lindex $region 2]
    set y1 [lindex $region 3]

    #koordinaten sortieren
    if { $x0 > $x1 } { set t $x0; set x0 $x1; set x1 $t }
    if { $y0 > $y1 } { set t $y0; set y0 $y1; set y1 $t }

    foreach id [$tkdraw(canvas) find withtag regsel] {
	#erstmal selektieren
	set bbox [tkdrawSelectItem $id]

	if { $bbox == "" } continue
	#object could not be selected

	set groups [tkdrawGetGroups $id]
	
	if { $groups != "" } {
	    #Gruppenobjekt
	    if { $x0 > [lindex $bbox 0] ||
		 $y0 > [lindex $bbox 1] ||
		 $x1 < [lindex $bbox 2] ||
		 $y1 < [lindex $bbox 3] }  {
		#item nich vollstaendig eingeschlossen
		tkdrawDeselectItem $id
	    }
	}
    }

    eval $tkdraw(canvas) dtag regsel regsel
}

proc tkdrawDeselectAll {} {
    global tkdraw
	
    #alle items deselektieren
    $tkdraw(canvas) dtag item selected
    $tkdraw(canvas) delete withtag selbox
    set tkdraw(firstselected) 1
    $tkdraw(canvas) dtag firstsel firstsel
    $tkdraw(canvas) dtag secsel secsel
}


proc tkdrawIsSelected { id } {
    global tkdraw

    if { [lsearch [$tkdraw(canvas) gettags $id] selected] != -1 } {
	return 1
    } else {
	return 0
    }
}
